package zander.library.media;

import java.awt.Component;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOError;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.apache.tika.Tika;

import zander.library.FileUtils;
import zander.ui.UIUtils;

public class ImageMediaData implements MediaData {
    private static final Tika TIKA = new Tika();

    private File sourceFile;

    private final SerializableImageContainer data;
    private final ImageMediaComponent viewer;
    private final String type;

    public ImageMediaData(File file) throws IOException {
        this(file, readFileBytes(file));
    }

    public ImageMediaData(byte[] data) throws IOException {
        this(FileUtils.createTempFile(data, true), data);
    }

    public ImageMediaData(File file, byte[] data) {
        try {
            ByteArrayInputStream in = new ByteArrayInputStream(data);
            BufferedImage img = UIUtils.fitImageToBox(ImageIO.read(in), UIUtils.IMAGE_PREVEW_SIZE);
            this.data = new SerializableImageContainer(img);
            if (!this.data.hasImage()) {
                throw new IllegalArgumentException("Image data was invalid");
            }
            viewer = new ImageMediaComponent(this.data.getImage());
            type = TIKA.detect(data);
            this.sourceFile = file;
        } catch (IOException e) {
            // Should not happen
            throw new IOError(e);
        }
    }

    @Override
    public byte[] loadRawBytes() throws IOException {
        return readFileBytes(sourceFile);
    }

    @Override
    public String getType() {
        return type;
    }

    @Override
    public Component createThumbnailComponent() {
        ImageMediaComponent thumbnail = new ImageMediaComponent(this.data.getImage());
        thumbnail.setPreferredSize(UIUtils.THUMBNAIL_SIZE);
        return thumbnail;
    }

    @Override
    public Component getViewerComponent() {
        return viewer;
    }

    @Override
    public File getSourceFile() {
        return sourceFile;
    }

    @Override
    public void setSourceFile(File file) {
        this.sourceFile = file;
    }

    @Override
    public boolean hasSourceFile() {
        return sourceFile != null;
    }

    private static byte[] readFileBytes(File file) throws IOException {
        FileInputStream in = new FileInputStream(file);
        byte[] buffer = in.readAllBytes();
        in.close();
        return buffer;
    }
}
