package zander.library.media;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import org.apache.tika.Tika;

import zander.library.FileUtils;
import zander.ui.ProgressTracker.ProgressManager;

public class MediaLoader {
    private static final Tika TIKA = new Tika();

    private static enum ContentType {
        VIDEO,
        AUDIO,
        IMAGE,
        UNKNOWN
    }

    public static MediaData loadFile(File file, boolean moveToTemp, ProgressManager t) throws IOException {
        FileInputStream in = new FileInputStream(file);
        byte[] data = in.readAllBytes();
        in.close();
        return loadBytes(file.getCanonicalFile(), data, moveToTemp, t);
    }

    public static MediaData loadBytes(File file, byte[] data, boolean moveToTemp, ProgressManager t) throws IOException {
        MediaData md = null;
        if (moveToTemp) {
            file = FileUtils.createTempFile(data, true);
        }
        switch (getFileType(data)) {
        case IMAGE:
            md = new ImageMediaData(file, data);
            break;
        case VIDEO:
            md = new VideoMediaData(file, data, t);
            break;
        case AUDIO:
            md = new AudioMediaData(file, data);
            break;
        default:
            md = new UnknownMediaData(file, data);
            break;
        }
        return md;
    }

    private static ContentType getFileType(byte[] data) {
        try {
            ByteArrayInputStream in = new ByteArrayInputStream(data);
            String type = TIKA.detect(in);
            if (type == null) {
                return ContentType.UNKNOWN;
            } else if (type.startsWith("image/")) {
                return ContentType.IMAGE;
            } else if (type.startsWith("video/")) {
                return ContentType.VIDEO;
            } else if (type.startsWith("audio/")) {
                return ContentType.AUDIO;
            } else {
                return ContentType.UNKNOWN;
            }
        } catch (IOException e) {
            return ContentType.UNKNOWN;
        }
    }
}
