package zander.library.media;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import javax.imageio.ImageIO;

public class SerializableImageContainer implements Serializable {
    private static final long serialVersionUID = 2606473417847123614L;

    private transient BufferedImage image;

    public SerializableImageContainer(BufferedImage image) {
        this.image = image;
    }

    public BufferedImage getImage() {
        return image;
    }

    public void setImage(BufferedImage image) {
        this.image = image;
    }

    public boolean hasImage() {
        return image != null;
    }

    private void writeObject(ObjectOutputStream out) throws IOException {
        out.defaultWriteObject();
        if (image == null) {
            out.writeBoolean(false);
        } else {
            out.writeBoolean(true);
            ImageIO.write(image, "png", out);
        }
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        if (in.readBoolean()) {
            image = ImageIO.read(in);
        } else {
            image = null;
        }
    }

}
