package zander.library.state;

import java.time.LocalDate;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.Date;

public class DateTimeUtils {

    private static final ArrayList<DateTimeFormatter> DATE_PARSERS = new ArrayList<DateTimeFormatter>();
    private static final ArrayList<DateTimeFormatter> TIME_PARSERS = new ArrayList<DateTimeFormatter>();
    static {
        DATE_PARSERS.add(DateTimeFormatter.ofPattern("MMM d[','] u"));
        DATE_PARSERS.add(DateTimeFormatter.ofPattern("MMMM d[','] u"));
        DATE_PARSERS.add(DateTimeFormatter.ofPattern("MMMMM d[','] u"));
        DATE_PARSERS.add(DateTimeFormatter.ofPattern("M'/'d'/'u"));
        DATE_PARSERS.add(DateTimeFormatter.ofPattern("M'-'d'-'u"));

        TIME_PARSERS.add(DateTimeFormatter.ofPattern("k':'m[':'s]"));
        TIME_PARSERS.add(DateTimeFormatter.ofPattern("h':'m[':'s] a"));
    }
    private static final DateTimeFormatter DATE_FORMATTER = DateTimeFormatter.ofPattern("MMMM d, uuuu");
    private static final DateTimeFormatter TIME_FORMATTER = DateTimeFormatter.ofPattern("hh':'mm[':'ss] a");

    public static LocalDate parseDate(String date) {
        if (date == null) {
            return null;
        }
        for (DateTimeFormatter f : DATE_PARSERS) {
            try {
                return LocalDate.parse(date, f);
            } catch (DateTimeParseException e) {
                continue;
            }
        }
        return null;
    }

    public static LocalTime parseTime(String time) {
        if (time == null) {
            return null;
        }
        for (DateTimeFormatter f : TIME_PARSERS) {
            try {
                return LocalTime.parse(time, f);
            } catch (DateTimeParseException e) {
                continue;
            }
        }
        return null;
    }

    public static String formatDate(LocalDate date) {
        if (date == null) {
            return null;
        }
        return date.format(DATE_FORMATTER);
    }

    public static String formatTime(LocalTime time) {
        if (time == null) {
            return null;
        }
        return time.format(TIME_FORMATTER);
    }

    public static LocalDate dateToLocalDate(Date date) {
        if (date == null) {
            return null;
        }
        return date.toInstant().atZone(ZoneId.of("GMT")).toLocalDate();
    }

    public static LocalTime dateToLocalTime(Date date) {
        if (date == null) {
            return null;
        }
        return date.toInstant().atZone(ZoneId.of("GMT")).toLocalTime().withSecond(0);
    }

    public static LocalTime cleanLocalTime(LocalTime time) {
        if (time == null) {
            return null;
        }
        return LocalTime.of(time.getHour(), time.getMinute(), 0);
    }

}
