package zander.library.state;

import java.util.ArrayList;
import java.util.List;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

public class LibraryConfig {

    private String name;
    private String license;
    private List<String> albums;

    public LibraryConfig() {
        this("", "", new ArrayList<String>());
    }

    public LibraryConfig(String name, String license, List<String> albums) {
        this.name = name;
        this.license = license;
        this.albums = albums;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getLicense() {
        return license;
    }

    public void setLicense(String license) {
        this.license = license;
    }

    public List<String> getAlbums() {
        return albums;
    }

    public void setAlbums(List<String> albums) {
        this.albums = albums;
    }

    public static LibraryConfig parseJson(String json) throws ParseException {
        JSONParser p = new JSONParser();
        Object o = p.parse(json);
        if (!(o instanceof JSONObject)) {
            throw new ClassCastException("Json root was not an object");
        }
        return parseJson((JSONObject) o);
    }

    public static LibraryConfig parseJson(JSONObject json) {
        Object nameObj = json.get("library_name");
        Object licenseObj = json.get("license");
        Object albumsObj = json.get("albums");
        if (!(nameObj instanceof String) || !(licenseObj instanceof String)) {
            throw new ClassCastException("Name and license must be strings");
        } else if (!(albumsObj instanceof JSONArray)) {
            throw new ClassCastException("Albums must be an array");
        }
        JSONArray albumsArr = (JSONArray) albumsObj;
        final ArrayList<String> albums = new ArrayList<String>(albumsArr.size());
        for (Object o : albumsArr) {
            if (!(o instanceof String)) {
                throw new ClassCastException("All members of albums must be a string");
            }
            albums.add((String) o);
        }
        return new LibraryConfig((String) nameObj, (String) licenseObj, albums);
    }

    @SuppressWarnings("unchecked")
    public static JSONObject toJson(LibraryConfig lc) {
        JSONObject root = new JSONObject();
        root.put("library_name", lc.getName());
        root.put("license", lc.getLicense());
        JSONArray albums = new JSONArray();
        albums.addAll(lc.getAlbums());
        root.put("albums", albums);
        return root;
    }

    public static String toJsonString(LibraryConfig lc) {
        return toJson(lc).toJSONString();
    }

}
