package zander.library.state;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

import zander.library.Library;
import zander.library.LocalCache;
import zander.library.state.AlbumConfig.AlbumImage;

public class LibraryState {

    private final Library library;

    private String name;
    private String license;
    private List<AlbumState> albums;

    private String originalName;
    private String originalLicense;
    private List<String> originalAlbums;

    public LibraryState(Library library) {
        this.library = library;
    }

    public Library getLibrary() {
        return library;
    }

    public List<AlbumState> getAlbums() {
        return albums;
    }

    public void setAlbums(List<AlbumState> albums) {
        this.albums = albums;
    }

    public String getLicense() {
        return license;
    }

    public void setLicense(String license) {
        this.license = license;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public AlbumState getAlbumForName(String name) {
        for (AlbumState a : albums) {
            if (a.getName().equals(name)) {
                return a;
            }
        }
        return null;
    }

    public AlbumState removeAlbumForName(String name) {
        for (int i = 0; i < albums.size(); ++i) {
            AlbumState a = albums.get(i);
            if (a.getName().equals(name)) {
                return albums.remove(i);
            }
        }
        return null;
    }

    public boolean hasAlbum(String name) {
        return getAlbumForName(name) != null;
    }

    public String getOriginalName() {
        return originalName;
    }

    public String getOriginalLicense() {
        return originalLicense;
    }

    public List<String> getOriginalAlbumNames() {
        return originalAlbums;
    }

    private static Double numToDouble(Number num) {
        if (num == null) {
            return null;
        }
        return num.doubleValue();
    }

    private static MediaState getMediaFromLibrary(LocalCache cache, String albumPath, AlbumImage ai) throws IOException {
        File imageFile = cache.forPath(albumPath + ai.name);
        long size = imageFile.length();
        MediaState state = new MediaState(ai.name, ai.description, DateTimeUtils.parseDate(ai.date), DateTimeUtils.parseTime(ai.time),
                                          numToDouble(ai.latitude), numToDouble(ai.longitude), size);
        return state;
    }

    private static AlbumState getAlbumFromLibrary(LibraryState parent, LibraryConfig libConf, String name) throws IOException, ParseException {
        LocalCache cache = parent.library.getLocalCache();
        String albumPath = Library.ALBUMS_FILE_PATH + "/" + name + "/";
        AlbumConfig config;
        if (cache.fileExists(albumPath + Library.ALBUM_CONFIG_FILE_NAME)) {
            byte[] data = cache.retrieveFile(albumPath + Library.ALBUM_CONFIG_FILE_NAME);
            config = AlbumConfig.parseJson(new String(data));
        } else {
            config = new AlbumConfig();
        }
        ArrayList<MediaState> media = new ArrayList<MediaState>();
        for (AlbumImage mn : config.getImages()) {
            media.add(getMediaFromLibrary(cache, albumPath, mn));
        }
        AlbumState state = new AlbumState(name, config.getTitle(), config.doesNameFollowTitle(), config.getThumnail(), media);
        return state;
    }

    public static LibraryState fromLibrary(Library library) throws IOException, ParseException {
        LibraryState state = new LibraryState(library);
        LocalCache cache = library.getLocalCache();
        LibraryConfig config;
        if (cache.fileExists(Library.CONFIG_FILE_PATH)) {
            byte[] data = cache.retrieveFile(Library.CONFIG_FILE_PATH);
            config = LibraryConfig.parseJson(new String(data));
        } else {
            config = new LibraryConfig();
        }
        state.name = config.getName();
        state.originalName = state.name;
        state.license = config.getLicense();
        state.originalLicense = state.license;
        state.albums = new ArrayList<AlbumState>();
        state.originalAlbums = new ArrayList<String>();
        for (String album : config.getAlbums()) {
            AlbumState as = getAlbumFromLibrary(state, config, album);
            state.albums.add(as);
            state.originalAlbums.add(as.getNameOnDisk());
        }
        return state;
    }

    public static JSONObject toJson(LibraryState state) {
        final ArrayList<String> albums = new ArrayList<String>();
        for (AlbumState m : state.albums) {
            albums.add(m.getName());
        }
        LibraryConfig lc = new LibraryConfig(state.name, state.license, albums);
        return LibraryConfig.toJson(lc);
    }

    public static String toJsonString(LibraryState state) {
        JSONObject jo = toJson(state);
        return jo.toJSONString();
    }

}
