package zander.ui.album;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.KeyEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.batik.ext.swing.GridBagConstants;

import zander.ui.UIUtils;
import zander.ui.docs.DocumentationViewer;

public class AlbumCreateDialog extends JDialog {
    private static final long serialVersionUID = 4903014837170670083L;

    public static final int RESPONSE_CANCEL = 0;
    public static final int RESPONSE_CREATE = 1;

    private final JLabel titleLabel;
    private final JTextField titleField;
    private final JLabel nameLabel;
    private final JTextField nameField;
    private final JCheckBox nameCheck;
    private final JPanel fieldPanel;
    private final JButton cancelButton;
    private final JButton helpButton;
    private final JButton createButton;
    private final JPanel buttonPanel;
    private final JPanel content;

    private int response = RESPONSE_CANCEL;

    public AlbumCreateDialog(Window parent) {
        super(parent, "Create Album");
        titleLabel = new JLabel("Album Title:");
        titleField = new JTextField(20);
        titleField.getDocument().addDocumentListener(new DocumentListener(){
                @Override
                public void insertUpdate(DocumentEvent e) { update(); }

                @Override
                public void removeUpdate(DocumentEvent e) { update(); }

                @Override
                public void changedUpdate(DocumentEvent e) { update(); }

                private void update() {
                    if (!nameCheck.isSelected()) {
                        nameField.setText(titleField.getText());
                    }
                }
            });
        nameLabel = new JLabel("Custom URL:");
        nameField = new JTextField(20);
        nameField.getDocument().addDocumentListener(new DocumentListener(){
                @Override
                public void insertUpdate(DocumentEvent e) { update(); }

                @Override
                public void removeUpdate(DocumentEvent e) { update(); }

                @Override
                public void changedUpdate(DocumentEvent e) { update(); }

                private void update() {
                    createButton.setEnabled(!nameField.getText().isBlank());
                }
            });
        nameLabel.setEnabled(false);
        nameField.setEnabled(false);
        nameCheck = new JCheckBox("Different Title and URL");
        nameCheck.addActionListener((e) -> {
                boolean dtu = nameCheck.isSelected();
                if (!dtu) {
                    nameField.setText(titleField.getText());
                }
                nameLabel.setEnabled(dtu);
                nameField.setEnabled(dtu);
            });
        fieldPanel = new JPanel(new GridBagLayout());
        buildFieldUI();
        cancelButton = new JButton("Cancel");
        cancelButton.setMnemonic(KeyEvent.VK_C);
        cancelButton.addActionListener((e) -> dispose());
        helpButton = new JButton("?");
        helpButton.addActionListener((e) -> DocumentationViewer.show(this, "Albums/Album Settings"));
        createButton = new JButton("Create");
        createButton.setMnemonic(KeyEvent.VK_R);
        createButton.addActionListener((e) -> {
                response = RESPONSE_CREATE;
                dispose();
            });
        createButton.setEnabled(false);
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(cancelButton);
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(helpButton);
        buttonPanel.add(createButton);
        content = new JPanel(new BorderLayout());
        content.add(fieldPanel, BorderLayout.CENTER);
        content.add(buttonPanel, BorderLayout.PAGE_END);
        setContentPane(content);
        pack();
        setResizable(false);
        setAlwaysOnTop(true);
        setModalityType(JDialog.DEFAULT_MODALITY_TYPE);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        UIUtils.centerWindow(this, parent);
    }

    public void buildFieldUI() {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridwidth = 1;
        gbc.gridheight = 1;
        gbc.insets = new Insets(1, 1, 1, 1);
        gbc.fill = GridBagConstraints.NONE;
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        fieldPanel.add(titleLabel, gbc);
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        fieldPanel.add(titleField, gbc);
        gbc.gridy = 1;
        gbc.gridx = 0;
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.LINE_END;
        fieldPanel.add(nameLabel, gbc);
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        fieldPanel.add(nameField, gbc);
        gbc.fill = GridBagConstants.NONE;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.gridwidth = 2;
        fieldPanel.add(nameCheck, gbc);
    }

    public int getResponse() {
        return response;
    }

    public String getAlbumName() {
        return nameField.getText();
    }

    public String getAlbumTitle() {
        return titleField.getText();
    }

    public boolean doesNameFollowTitle() {
        return !nameCheck.isSelected();
    }

}
