package zander.ui.album;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.DropMode;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.TransferHandler;
import javax.swing.UIManager;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import zander.library.LibraryMediaCache;
import zander.library.state.AlbumState;
import zander.library.state.LibraryState;
import zander.library.state.MediaState;
import zander.ui.LibraryFrame;
import zander.ui.LibraryUploadDialog;
import zander.ui.ProgressTracker;
import zander.ui.UIUtils;
import zander.ui.cellrenderer.AlbumCellRenderer;
import zander.ui.docs.DocumentationViewer;
import zander.ui.media.MediaManagePanel;
import zander.ui.transferhandlers.ListArrangeTransferHandler;

public class AlbumSelectPanel extends JPanel {
    private static final long serialVersionUID = -3560698858822813839L;

    public static final int ALBUMS_TO_HOLD_IN_MEMORY = 5;

    private final LibraryFrame parent;
    private final LibraryState library;
    private final LibraryMediaCache cache;
    private final JPopupMenu popupMenu;
    private final JMenuItem openItem;
    private final JMenuItem deleteItem;
    private final JMenuItem newItem;
    private final JMenuItem editItem;
    private final JButton backButton;
    private final JButton libraryPropButton;
    private final JLabel searchLabel;
    private final JTextField searchField;
    private final JButton uploadButton;
    private final JButton helpButton;
    private final JButton newButton;
    private final JButton deleteButton;
    private final JButton openButton;
    private final JButton editButton;
    private final JPanel buttonPanel;
    private final JLabel screenLabel;
    private final JPanel topPanel;
    private final DefaultListModel<AlbumState> albumModel;
    private final TransferHandler albumTransferHandler;
    private final JList<AlbumState> albumList;
    private final JScrollPane albumScroll;
    private final AlbumPropertyPanel propsPanel;
    private final JLabel manyPropsLabel;
    private final JLabel noPropsLabel;
    private final JSplitPane splitPane;

    private int lastClicked = -1;
    private Component currentComp;
    private boolean firstToggle = true;
    private boolean canceled = false;
    private final List<AlbumState> openHistory;

    public AlbumSelectPanel(LibraryFrame parent, List<AlbumState> openHistory, String localName) {
        super(new BorderLayout());
        this.parent = parent;
        this.openHistory = openHistory;
        this.library = parent.getLibraryState();
        cache = parent.getMediaCache();
        popupMenu = new JPopupMenu();
        openItem = new JMenuItem("Open");
        openItem.setEnabled(false);
        openItem.addActionListener(this::openAlbumHandler);
        deleteItem = new JMenuItem("Delete");
        deleteItem.setEnabled(false);
        deleteItem.addActionListener(this::deleteAlbumHandler);
        newItem = new JMenuItem("New");
        newItem.addActionListener((e) -> newAlbum());
        editItem = new JMenuItem("Toggle Property Panel");
        editItem.addActionListener(this::toggleEditPanelHandler);
        popupMenu.add(openItem);
        popupMenu.add(deleteItem);
        popupMenu.addSeparator();
        popupMenu.add(newItem);
        popupMenu.add(editItem);
        backButton = new JButton("Back to Libraries");
        backButton.setMnemonic(KeyEvent.VK_B);
        backButton.addActionListener((e) -> parent.backButtonHandler());
        libraryPropButton = new JButton("Library Properties");
        libraryPropButton.setMnemonic(KeyEvent.VK_L);
        libraryPropButton.addActionListener((e) -> {
                LibraryPropertyDialog diag = new LibraryPropertyDialog(SwingUtilities.getWindowAncestor(this), library);
                diag.setVisible(true);
                if (diag.getResponse() == LibraryPropertyDialog.RESPONSE_SAVE) {
                    library.setName(diag.getLibraryName());
                    library.setLicense(diag.getLibraryLicense());
                }
            });
        searchLabel = new JLabel("Search:");
        searchField = new JTextField(10);
        searchField.getDocument().addDocumentListener(new DocumentListener() {
                @Override
                public void insertUpdate(DocumentEvent e) { update(); }

                @Override
                public void removeUpdate(DocumentEvent e) { update(); }

                @Override
                public void changedUpdate(DocumentEvent e) { update(); }

                public void update() {
                    doSearch(searchField.getText());
                }
            });
        uploadButton = new JButton("Upload");
        uploadButton.setMnemonic(KeyEvent.VK_U);
        uploadButton.addActionListener((e) -> {
            LibraryUploadDialog diag = new LibraryUploadDialog(SwingUtilities.getWindowAncestor(this));
            if (diag.upload(library, cache)) {
                parent.handleUploadDone(this);
            }
        });
        helpButton = new JButton("?");
        helpButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (currentComp == splitPane) {
                    DocumentationViewer.show(AlbumSelectPanel.this, "Albums/Album Settings");
                } else {
                    DocumentationViewer.show(AlbumSelectPanel.this, "Albums/Managing Albums");
                }
            }
        });
        newButton = new JButton("New");
        newButton.setMnemonic(KeyEvent.VK_N);
        newButton.addActionListener((e) -> newAlbum());
        deleteButton = new JButton("Delete");
        deleteButton.setMnemonic(KeyEvent.VK_D);
        deleteButton.setEnabled(false);
        deleteButton.addActionListener(this::deleteAlbumHandler);
        openButton = new JButton("Open");
        openButton.setMnemonic(KeyEvent.VK_O);
        openButton.setEnabled(false);
        openButton.addActionListener(this::openAlbumHandler);
        editButton = new JButton("Toggle Property Panel");
        editButton.setMnemonic(KeyEvent.VK_T);
        editButton.addActionListener(this::toggleEditPanelHandler);
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(backButton);
        buttonPanel.add(libraryPropButton);
        buttonPanel.add(searchLabel);
        buttonPanel.add(searchField);
        buttonPanel.add(uploadButton);
        buttonPanel.add(helpButton);
        buttonPanel.add(newButton);
        buttonPanel.add(deleteButton);
        buttonPanel.add(openButton);
        buttonPanel.add(editButton);
        screenLabel = new JLabel(localName, SwingConstants.CENTER);
        screenLabel.setFont(UIUtils.scaleFont(UIManager.getFont("Label.font"), 2.0f));
        topPanel = new JPanel(new BorderLayout());
        topPanel.add(buttonPanel, BorderLayout.CENTER);
        topPanel.add(screenLabel, BorderLayout.PAGE_END);
        albumModel = new DefaultListModel<AlbumState>();
        albumTransferHandler = new ListArrangeTransferHandler(new Runnable() {
            @Override
            public void run() {
                propsPanel.updateName();
            };
        }, () -> {
            library.getAlbums().clear();
            for (int i = 0; i < albumModel.size(); ++i) {
                library.getAlbums().add(albumModel.get(i));
            }
        });
        albumList = new JList<AlbumState>(albumModel);
        albumList.setDropMode(DropMode.INSERT);
        albumList.setDragEnabled(true);
        albumList.setLayoutOrientation(JList.HORIZONTAL_WRAP);
        albumList.setVisibleRowCount(-1);
        albumList.setCellRenderer(new AlbumCellRenderer(cache));
        albumList.setTransferHandler(albumTransferHandler);
        albumList.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if (e.getButton() == MouseEvent.BUTTON1) {
                    Rectangle r = albumList.getCellBounds(0, albumList.getLastVisibleIndex());
                    if (r != null && r.contains(e.getPoint())) {
                        if (e.getClickCount() == 1) {
                            int i = albumList.locationToIndex(e.getPoint());
                            lastClicked = i;
                        } else if (e.getClickCount() == 2) {
                            int i = albumList.locationToIndex(e.getPoint());
                            openAlbum(i);
                        }
                    }
                } else if (e.getButton() == MouseEvent.BUTTON3) {
                    Rectangle r = albumList.getCellBounds(0, albumList.getLastVisibleIndex());
                    if (r != null && r.contains(e.getPoint())) {
                        int i = albumList.locationToIndex(e.getPoint());
                        if ((e.getModifiersEx() & InputEvent.CTRL_DOWN_MASK) == InputEvent.CTRL_DOWN_MASK) {
                            albumList.addSelectionInterval(i, i);
                        } else if (lastClicked != -1
                                && (e.getModifiersEx() & InputEvent.SHIFT_DOWN_MASK) == InputEvent.SHIFT_DOWN_MASK) {
                            albumList.setSelectionInterval(lastClicked, i);
                        } else if (!albumList.isSelectedIndex(i)) {
                            albumList.setSelectedIndex(i);
                        }
                    }
                    int numSel = albumList.getSelectedIndices().length;
                    openItem.setEnabled(numSel == 1);
                    deleteItem.setEnabled(numSel > 0);
                    popupMenu.show(albumList, e.getX(), e.getY());
                }
            }
        });
        albumScroll = new JScrollPane(albumList);
        propsPanel = new AlbumPropertyPanel(cache, openHistory);
        propsPanel.addAlbumPropertyUpdateListener((e) -> {
                revalidate();
                repaint();
            });
        manyPropsLabel = new JLabel("Multiple items selected.", JLabel.CENTER);
        noPropsLabel = new JLabel("No items selected.", JLabel.CENTER);
        splitPane = new JSplitPane();
        splitPane.setRightComponent(noPropsLabel);
        splitPane.setOneTouchExpandable(false);
        add(topPanel, BorderLayout.PAGE_START);
        add(albumScroll, BorderLayout.CENTER);
        currentComp = albumScroll;
        albumList.addListSelectionListener((e) -> refreshAlbums());
        loadMedia();
    }

    private void loadMedia() {
        ProgressTracker pt = new ProgressTracker(null, "Loading Thumbnails", "Loading thumbnails...", (t) -> {
                t.setMinimum(0);
                t.setMaximum(library.getAlbums().size());
                t.setIndeterminate(false);
                for (AlbumState album : library.getAlbums()) {
                    cache.getMedia(album.getName(), album.getThumbnailName(), t);
                    albumModel.addElement(album);
                    t.setProgress(t.getProgress() + 1);
                    if (t.isCanceled()) {
                        canceled = true;
                        break;
                    }
                }
        });
        pt.start();
    }

    public boolean wasCancled() {
        return canceled;
    }

    private void toggleEditPanelHandler(ActionEvent e) {
        if (firstToggle) {
            splitPane.setDividerLocation((int) (getWidth() / 2));
            firstToggle = false;
        }
        if (currentComp == albumScroll) {
            int div = splitPane.getDividerLocation();
            remove(albumScroll);
            add(splitPane, BorderLayout.CENTER);
            currentComp = splitPane;
            splitPane.setLeftComponent(albumScroll);
            splitPane.setDividerLocation(div);
        } else {
            remove(splitPane);
            add(albumScroll, BorderLayout.CENTER);
            currentComp = albumScroll;
        }
        revalidate();
    }

    private void refreshAlbums() {
        int numSel = albumList.getSelectedIndices().length;
        if (numSel == 0) {
            int div = splitPane.getDividerLocation();
            splitPane.setRightComponent(noPropsLabel);
            splitPane.setDividerLocation(div);
        } else if (numSel == 1) {
            if (splitPane.getRightComponent() != propsPanel) {
                int div = splitPane.getDividerLocation();
                splitPane.setRightComponent(propsPanel);
                splitPane.setDividerLocation(div);
            }
            propsPanel.setAlbum(albumList.getSelectedValue());
        } else if (splitPane.getRightComponent() != manyPropsLabel) {
            int div = splitPane.getDividerLocation();
            splitPane.setRightComponent(manyPropsLabel);
            splitPane.setDividerLocation(div);
        }
        revalidate();
        openButton.setEnabled(numSel == 1);
        deleteButton.setEnabled(numSel > 0);
    }

    public void rereadAlbum() {
        AlbumState as = albumList.getSelectedValue();
        if (as != null) {
            propsPanel.setAlbum(albumList.getSelectedValue(), true);
            repaint();
            revalidate();
        }
    }

    public void reloadAlbumsFromLibrary() {
        doSearch(searchField.getText());
    }

    private void openAlbumHandler(ActionEvent e) {
        int i = albumList.getSelectedIndex();
        if (i != -1) {
            openAlbum(i);
        }
    }

    private void openAlbum(int i) {
        albumList.setSelectedIndex(i);
        AlbumState a = albumModel.get(i);
        MediaManagePanel p = new MediaManagePanel(parent, a);
        openHistory.remove(a);
        openHistory.add(a);
        while (openHistory.size() > ALBUMS_TO_HOLD_IN_MEMORY) {
            AlbumState toSwap = openHistory.remove(0);
            cache.swapOutAlbum(toSwap.getName(), false);
        }
        if (!p.wasCanceled()) {
            parent.setCurrentPanel(p);
        }
    }

    private void deleteAlbumHandler(ActionEvent e) {
        List<AlbumState> as = albumList.getSelectedValuesList();
        int response;
        if (as.size() == 1) {
            response = JOptionPane.showConfirmDialog(this, "Are you sure you want to delete the album '" + as.get(0).getName() + "'?",
                                          "Confirm Delete", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
        } else {
            response = JOptionPane.showConfirmDialog(this, "Are you sure you want to delete " + as.size() + " albums?",
                                          "Confirm Delete", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
        }
        if (response == JOptionPane.YES_OPTION) {
            for (AlbumState a : as) {
                deleteAlbum(a);
            }
        }
    }

    private void deleteAlbum(AlbumState a) {
        albumModel.removeElement(a);
        library.removeAlbumForName(a.getName());
        cache.removeAlbum(a.getName());
        openHistory.remove(a);
    }

    private void newAlbum() {
        AlbumCreateDialog diag = new AlbumCreateDialog(SwingUtilities.getWindowAncestor(this));
        diag.setVisible(true);
        if (diag.getResponse() == AlbumCreateDialog.RESPONSE_CREATE) {
            String name = diag.getAlbumName();
            if (library.getAlbumForName(name) != null) {
                JOptionPane.showMessageDialog(this, "An album with the name '" + name + "' already exists!",
                                              "Name Conflict", JOptionPane.ERROR_MESSAGE);
            } else {
                AlbumState na = new AlbumState(name, diag.getAlbumTitle(), diag.doesNameFollowTitle(),
                                               "", new ArrayList<MediaState>());
                na.setNameOnDisk(null);
                library.getAlbums().add(na);
                albumModel.addElement(na);
                openAlbum(albumModel.getSize() - 1);
            }
        }
    }

    private void doSearch(String search) {
        if (!search.isBlank()) {
            albumList.setDragEnabled(false);
            albumList.setTransferHandler(null);
        } else {
            albumList.setDragEnabled(true);
            albumList.setTransferHandler(albumTransferHandler);
        }
        search = search.trim();
        albumModel.clear();
        for (AlbumState a : library.getAlbums()) {
            if (a.getTitle().trim().startsWith(search)) {
                albumModel.addElement(a);
            }
        }
    }
}
