package zander.ui.album;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import zander.library.LibraryMediaCache;
import zander.library.state.AlbumState;
import zander.library.state.LibraryState;
import zander.library.state.MediaState;
import zander.ui.ProgressTracker;
import zander.ui.UIUtils;
import zander.ui.cellrenderer.MediaCellRenderer;

public class AlbumThumbnailSelectDialog extends JDialog {
    private static final long serialVersionUID = -9035406003694248528L;

    public static final int RESPONSE_CANCEL = 0;
    public static final int RESPONSE_SELECT = 1;

    private int response = RESPONSE_CANCEL;

    private final LibraryMediaCache cache;
    private final LibraryState library;
    private final AlbumState album;
    private final List<AlbumState> openHistory;

    private final DefaultListModel<MediaState> listModel;
    private final JList<MediaState> thumbList;
    private final JScrollPane scroll;
    private final JButton cancelButton;
    private final JButton selectButton;
    private final JPanel buttonPanel;
    private final JPanel content;

    private boolean valid = true;

    public AlbumThumbnailSelectDialog(Window parent, LibraryMediaCache cache, AlbumState album, List<AlbumState> openHistory) {
        super(parent, "Select Thumbnail");
        this.cache = cache;
        this.library = cache.getLibrary();
        this.album = album;
        this.openHistory = openHistory;
        listModel = new DefaultListModel<MediaState>();
        thumbList = new JList<MediaState>(listModel);
        thumbList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        thumbList.setLayoutOrientation(JList.HORIZONTAL_WRAP);
        thumbList.setVisibleRowCount(-1);
        thumbList.setCellRenderer(new MediaCellRenderer(cache, album));
        thumbList.setSelectedValue(album.getThumbnailName(), true);
        thumbList.addMouseListener(new MouseAdapter() {
                @Override
                public void mouseClicked(MouseEvent e) {
                    if (e.getButton() == MouseEvent.BUTTON1) {
                        Rectangle r = thumbList.getCellBounds(0, thumbList.getLastVisibleIndex());
                        if (r != null && r.contains(e.getPoint())) {
                            if (e.getClickCount() == 2) {
                                int i = thumbList.locationToIndex(e.getPoint());
                                thumbList.setSelectedIndex(i);
                                response = RESPONSE_SELECT;
                                dispose();
                            }
                        }
                    }
                }
            });
        scroll = new JScrollPane(thumbList);
        cancelButton = new JButton("Canel");
        cancelButton.setMnemonic(KeyEvent.VK_C);
        cancelButton.addActionListener((e) -> {
                dispose();
            });
        selectButton = new JButton("Select");
        selectButton.setMnemonic(KeyEvent.VK_S);
        selectButton.addActionListener((e) -> {
                response = RESPONSE_SELECT;
                dispose();
            });
        thumbList.addListSelectionListener((e) -> {
                if (thumbList.getSelectedIndex() == -1) {
                    selectButton.setEnabled(false);
                } else {
                    selectButton.setEnabled(true);
                }
            });
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(cancelButton);
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(selectButton);
        content = new JPanel(new BorderLayout());
        content.add(scroll, BorderLayout.CENTER);
        content.add(buttonPanel, BorderLayout.PAGE_END);
        setContentPane(content);
        if (parent != null) {
            setSize((2 * parent.getWidth()) / 3, (2 * parent.getHeight()) / 3);
        } else {
            pack();
        }
        setMinimumSize(new Dimension(content.getPreferredSize().width, 200));
        UIUtils.centerWindow(this, parent);
        setResizable(true);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setAlwaysOnTop(true);
        setModalityType(JDialog.DEFAULT_MODALITY_TYPE);
        loadMedia();
    }

    private void loadMedia() {
        ProgressTracker pt = new ProgressTracker(null, "Loading Media", "Loading media...", (t) -> {
            t.setIndeterminate(false);
            t.setMinimum(0);
            t.setMaximum(album.getMedia().size());
            for (MediaState media : album.getMedia()) {
                cache.getMedia(album.getName(), media.getName(), t);
                t.setProgress(t.getProgress() + 1);
                listModel.addElement(media);
                if (t.isCanceled()) {
                    valid = false;
                    break;
                }
            }
            openHistory.remove(album);
            openHistory.add(album);
            while (openHistory.size() > AlbumSelectPanel.ALBUMS_TO_HOLD_IN_MEMORY) {
                AlbumState toSwap = openHistory.remove(0);
                cache.swapOutAlbum(toSwap.getName(), false);
            }
        });
        pt.start();
    }

    @Override
    public void setVisible(boolean visible) {
        if (valid) {
            super.setVisible(visible);
        }
    }

    public LibraryMediaCache getMediaCahce() {
        return cache;
    }

    public LibraryState getLibrary() {
        return library;
    }

    public AlbumState getAlbum() {
        return album;
    }

    public String getSelectedThumbnail() {
        return thumbList.getSelectedValue().getName();
    }

    public int getResponse () {
        return response;
    }

}
