package zander.ui.library;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Set;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;

import zander.library.secrets.SecretsFactory;
import zander.ui.FileChooserField;
import zander.ui.UIUtils;
import zander.ui.docs.DocumentationViewer;

public class CuratorOptionsDialog extends JDialog {
    private static final long serialVersionUID = -1839450524327574007L;

    public static final String NO_DEFAULT_LIBRARY = "Do not automatically open any libraries";
    public static final String LAST_DEFAULT_LIBRARY = "Automatically open the last oppened library";

    public static final int RESPONSE_CANCEL = 0;
    public static final int RESPONSE_SAVE = 1;

    private int response = RESPONSE_CANCEL;

    private final JLabel defaultPathLabel;
    private final FileChooserField defaultPathField;
    private final JLabel passStoreLabel;
    private final JComboBox<SecretsFactory> passStoreBox;
    private final JLabel lafLabel;
    private final JComboBox<String> lafBox;
    private final JLabel defaultLibraryLabel;
    private final DefaultComboBoxModel<Object> defaultLibraryModel;
    private final JComboBox<Object> defaultLibraryBox;
    private final JLabel openWebLabel;
    private final JComboBox<String> openWebBox;
    private final JLabel askToBackLabel;
    private final JCheckBox askToBackCheck;
    private final JPanel settingsPanel;
    private final JButton cancelButton;
    private final JButton helpButton;
    private final JButton saveButton;
    private final JPanel buttonPanel;
    private final JPanel content;

    public CuratorOptionsDialog(Window parent) {
        super(parent, "Curator Settings");
        defaultPathLabel = new JLabel("Default Library Location:");
        defaultPathField = new FileChooserField(20, LibrarySelectFrame.DEFAULT_LIBRARY_PATH, new File(LibrarySelectFrame.DEFAULT_LIBRARY_PATH));
        defaultPathField.getChooser().setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        defaultPathField.getChooser().setDialogTitle("Select Default Library Location");
        passStoreLabel = new JLabel("Password Store Backend:");
        passStoreBox = new JComboBox<SecretsFactory>(getPasswordBackends());
        passStoreBox.setSelectedItem(LibrarySelectFrame.SECRETS_FACTORY);
        lafLabel = new JLabel("Theme:");
        lafBox = new JComboBox<String>(getLookAndFeels());
        setInitialLookAndFeel();
        defaultLibraryLabel = new JLabel("Default Library:");
        defaultLibraryModel = new DefaultComboBoxModel<Object>(LibrarySelectFrame.LIBRARY_ENTRIES.toArray());
        defaultLibraryModel.insertElementAt(NO_DEFAULT_LIBRARY, 0);
        defaultLibraryModel.insertElementAt(LAST_DEFAULT_LIBRARY, 1);
        defaultLibraryBox = new JComboBox<Object>(defaultLibraryModel);
        defaultLibraryBox.setSelectedItem(LibrarySelectFrame.DEFAULT_LIBRARY);
        openWebLabel = new JLabel("Default Upload Action:");
        openWebBox = new JComboBox<String>(new String[] {"Always ask", "Never open library", "Always open library"});
        openWebBox.setSelectedIndex(LibrarySelectFrame.LIBRARY_WEB_HANDLER);
        askToBackLabel = new JLabel("Ask Before Exiting Album:");
        askToBackCheck = new JCheckBox();
        askToBackCheck.setSelected(LibrarySelectFrame.ASK_TO_RETURN_TO_LIBRARY_SELECT);
        settingsPanel = new JPanel(new GridBagLayout());
        buildSettingsPanel();
        cancelButton = new JButton("Cancel");
        cancelButton.setMnemonic(KeyEvent.VK_C);
        cancelButton.addActionListener((e) -> {
                dispose();
            });
        helpButton = new JButton("?");
        helpButton.addActionListener((e) -> DocumentationViewer.show(this, "Libraries/Global Settings"));
        saveButton = new JButton("Save");
        saveButton.setMnemonic(KeyEvent.VK_S);
        saveButton.addActionListener((e) -> {
                dispose();
                response = RESPONSE_SAVE;
            });
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(cancelButton);
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(helpButton);
        buttonPanel.add(saveButton);
        content = new JPanel(new BorderLayout());
        content.add(settingsPanel, BorderLayout.CENTER);
        content.add(buttonPanel, BorderLayout.PAGE_END);
        setContentPane(content);
        setModalityType(JDialog.DEFAULT_MODALITY_TYPE);
        setAlwaysOnTop(true);
       setResizable(false);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        pack();
        UIUtils.centerWindow(this, parent);
    }

    private void setInitialLookAndFeel() {
        if (LibrarySelectFrame.LOOK_AND_FEEL.equals(UIUtils.SYSTEM_LAF_NAME)) {
            lafBox.setSelectedIndex(0);
        } else {
            lafBox.setSelectedItem(LibrarySelectFrame.LOOK_AND_FEEL);
        }
	}

	private String[] getLookAndFeels() {
        final ArrayList<String> lafs = new ArrayList<String>();
        for (LookAndFeelInfo laf : UIManager.getInstalledLookAndFeels()) {
            if (laf.getName().equals(UIUtils.SYSTEM_LAF_NAME)) {
                lafs.add(0, "Default (" + laf.getName() + ")");
            } else {
                lafs.add(laf.getName());
            }
        }
        return lafs.toArray(new String[0]);
    }

    private SecretsFactory[] getPasswordBackends() {
        Set<SecretsFactory> sfs = SecretsFactory.getFactories();
        ArrayList<SecretsFactory> backends = new ArrayList<SecretsFactory>();
        backends.add(SecretsFactory.getDefaultFactory());
        for (SecretsFactory sf : sfs) {
            if (sf.isSupported()) {
                backends.add(sf);
            }
        }
        return backends.toArray(new SecretsFactory[0]);
    }

    private void buildSettingsPanel() {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(1, 1, 1, 1);
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.LINE_END;
        gbc.gridy = 0;
        gbc.gridx = 0;
        settingsPanel.add(defaultPathLabel, gbc);
        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.LINE_START;
        settingsPanel.add(defaultPathField, gbc);
        gbc.gridy = 1;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        settingsPanel.add(passStoreLabel, gbc);
        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.LINE_START;
        settingsPanel.add(passStoreBox, gbc);
        gbc.gridy = 2;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        settingsPanel.add(lafLabel, gbc);
        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.LINE_START;
        settingsPanel.add(lafBox, gbc);
        gbc.gridy = 3;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        settingsPanel.add(defaultLibraryLabel, gbc);
        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.LINE_START;
        settingsPanel.add(defaultLibraryBox, gbc);
        gbc.gridy = 4;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        settingsPanel.add(openWebLabel, gbc);
        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.LINE_START;
        settingsPanel.add(openWebBox, gbc);
        gbc.gridy = 5;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        settingsPanel.add(askToBackLabel, gbc);
        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.LINE_START;
        settingsPanel.add(askToBackCheck, gbc);
    }

    public int getResponse() {
        return response;
    }

    private String getSelectedLafString() {
        if (lafBox.getSelectedIndex() == 0) {
            return UIUtils.SYSTEM_LAF_NAME;
        }
        return lafBox.getItemAt(lafBox.getSelectedIndex());
    }

    public void updateSettigs() {
        LibrarySelectFrame.updateLibraryPath(defaultPathField.getPathField().getText());
        if (!passStoreBox.getSelectedItem().equals(LibrarySelectFrame.SECRETS_FACTORY)) {
            LibrarySelectFrame.updateSecretsFactory(passStoreBox.getItemAt(passStoreBox.getSelectedIndex()));
        }
        LibrarySelectFrame.updateLookAndFeel(getSelectedLafString());
        LibrarySelectFrame.updateDefaultLibrarySelection(defaultLibraryBox.getSelectedItem());
        LibrarySelectFrame.updateWebOpenHandler(openWebBox.getSelectedIndex());
        LibrarySelectFrame.updateAskToBack(askToBackCheck.isSelected());
    }

}
