package zander.ui.library;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.KeyEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import zander.ui.FileChooserField;
import zander.ui.UIUtils;
import zander.ui.docs.DocumentationViewer;

public class LibraryImportDialog extends JDialog {
    private static final long serialVersionUID = 8184071881411029915L;

    public static final int RESPONSE_CANCEL = 0;
    public static final int RESPONSE_IMPORT = 1;

    private int response = RESPONSE_CANCEL;

    private final JLabel urlLabel;
    private final JTextField urlField;
    private final JLabel ftpLabel;
    private final JTextField ftpField;
    private final JLabel usernameLabel;
    private final JTextField usernameField;
    private final JLabel passwordLabel;
    private final JPasswordField passwordField;
    private final JLabel ftpModeLabel;
    private final JComboBox<String> ftpModeCombo;
    private final JLabel fileLabel;
    private final FileChooserField fileField;
    private final JPanel entryPanel;
    private final JButton cancelButton;
    private final JButton helpButton;
    private final JButton importButton;
    private final JPanel buttonPanel;
    private final JPanel content;

    public LibraryImportDialog(Window parent) {
        super(parent, "Import Library");
        urlLabel = new JLabel("(Optional) URL:");
        urlField = new JTextField(10);
        ftpLabel = new JLabel("FTP:");
        ftpField = new JTextField(10);
        usernameLabel = new JLabel("Username:");
        usernameField = new JTextField(10);
        passwordLabel = new JLabel("Password:");
        passwordField = new JPasswordField(10);
        ftpModeLabel = new JLabel("FTP Mode:");
        ftpModeCombo = new JComboBox<String>(new String[] { "Passive", "Active" });
        fileLabel = new JLabel("Source:");
        fileField = new FileChooserField(10);
        fileField.getChooser().setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fileField.getChooser().setMultiSelectionEnabled(false);
        fileField.getChooser().setDialogType(JFileChooser.OPEN_DIALOG);
        fileField.getChooser().setDialogTitle("Select Library Location");
        entryPanel = new JPanel(new GridBagLayout());
        buildEntryPanel();
        cancelButton = new JButton("Cancel");
        cancelButton.setMnemonic(KeyEvent.VK_C);
        cancelButton.addActionListener((e) -> {
            dispose();
        });
        helpButton = new JButton("?");
        helpButton.addActionListener((e) -> DocumentationViewer.show(this, "Libraries/Local Library Settings"));
        importButton = new JButton("Import");
        importButton.setMnemonic(KeyEvent.VK_I);
        importButton.setEnabled(false);
        importButton.addActionListener((e) -> {
            dispose();
            response = RESPONSE_IMPORT;
        });
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(cancelButton);
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(helpButton);
        buttonPanel.add(importButton);
        final DocumentListener formFillListener = new DocumentListener(){
            @Override
            public void insertUpdate(DocumentEvent e) {
                update();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                update();
            }


            @Override
            public void changedUpdate(DocumentEvent e) {
                update();
            }

            private void update() {
                importButton.setEnabled(!ftpField.getText().isBlank() && !usernameField.getText().isBlank() && !fileField.getPathField().getText().isBlank());
            }
        };
        ftpField.getDocument().addDocumentListener(formFillListener);
        usernameField.getDocument().addDocumentListener(formFillListener);
        fileField.getPathField().getDocument().addDocumentListener(formFillListener);
        content = new JPanel(new BorderLayout());
        content.add(entryPanel, BorderLayout.CENTER);
        content.add(buttonPanel, BorderLayout.PAGE_END);
        setContentPane(content);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setModalityType(JDialog.DEFAULT_MODALITY_TYPE);
        setAlwaysOnTop(true);
        setResizable(false);
        pack();
        UIUtils.centerWindow(this, parent);
    }

    private void buildEntryPanel() {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridwidth = 1;
        gbc.gridheight = 1;
        gbc.insets = new Insets(1, 1, 1, 1);
        gbc.fill = GridBagConstraints.NONE;
        gbc.gridy = 0;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        entryPanel.add(urlLabel, gbc);
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        entryPanel.add(urlField, gbc);
        gbc.gridy = 1;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        gbc.fill = GridBagConstraints.NONE;
        entryPanel.add(ftpLabel, gbc);
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        entryPanel.add(ftpField, gbc);
        gbc.gridy = 2;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        gbc.fill = GridBagConstraints.NONE;
        entryPanel.add(usernameLabel, gbc);
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        entryPanel.add(usernameField, gbc);
        gbc.gridy = 3;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        gbc.fill = GridBagConstraints.NONE;
        entryPanel.add(passwordLabel, gbc);
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        entryPanel.add(passwordField, gbc);
        gbc.gridy = 4;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        gbc.fill = GridBagConstraints.NONE;
        entryPanel.add(ftpModeLabel, gbc);
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        entryPanel.add(ftpModeCombo, gbc);
        gbc.gridy = 5;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        gbc.fill = GridBagConstraints.NONE;
        entryPanel.add(fileLabel, gbc);
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        entryPanel.add(fileField, gbc);
    }

    public int getResponse() {
        return response;
    }

    public String getURL() {
        return urlField.getText();
    }

    public String getFTP() {
        return ftpField.getText();
    }

    public String getUsername() {
        return usernameField.getText();
    }

    public String getPassword() {
        return new String(passwordField.getPassword());
    }

    public String getFile() {
        return fileField.getPathField().getText();
    }

    public boolean isFtpPassiveMode() {
        return ftpModeCombo.getSelectedItem().equals("Passive");
    }

}
