package zander.ui.media.map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LocationEncoderCacheCleaner {
    private static final Logger LOGGER = LoggerFactory.getLogger(LocationEncoderCacheCleaner.class);

    private final LocationEncoder encoder;
    /** in miliiseconds */
    private final long frequency;
    private final Thread cleanThread;
    private boolean running = true;
    private boolean paused = false;
    private long lastClean = 0;

    public LocationEncoderCacheCleaner(LocationEncoder encoder, long milis) {
        this.encoder = encoder;
        frequency = milis;
        cleanThread = new Thread(this::cleanLoop);
        cleanThread.start();
    }

    public void dispose() {
        running = false;
    }

    public boolean isDisposed() {
        return !running;
    }

    public void pause() {
        paused = true;
    }

    public boolean isPaused() {
        return paused;
    }

    public void resume() {
        paused = false;
    }

    public long getFrequency() {
        return frequency;
    }

    public LocationEncoder getEncoder() {
        return encoder;
    }

    private void cleanLoop() {
        LOGGER.info("Starting location cache clean loop with frequency of {}ms", frequency);
        while (running) {
            if (!paused && lastClean + frequency <= System.currentTimeMillis()) {
                LOGGER.info("Cleaning location cache...");
                encoder.cleanOldCacheEntries();
                LOGGER.info("Done cleaning location cache");
                lastClean = System.currentTimeMillis();
            }
            try {
                Thread.sleep(5);
            } catch (InterruptedException e) {
                // ignore
            }
        }
        LOGGER.info("Stopped location cache clean loop!");
    }

}
