package zander.ui.media.map;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTextArea;

public class LocationEntryComponent extends JPanel {
    private static final long serialVersionUID = 123157372342348205L;

    private final MouseAdapter actionMouseAdapter  = new MouseAdapter() {
        @Override
        public void mouseClicked(MouseEvent e) {
            fireActionEvent();
        }
    };
    private final ArrayList<ActionListener> actionListeners = new ArrayList<ActionListener>();
    private final LocationEntry entry;

    private final JTextArea nameArea;
    private final JTextArea coordArea;

    public LocationEntryComponent(LocationEntry entry) {
        this.entry = entry;
        nameArea = new JTextArea(entry.getName());
        nameArea.setEnabled(false);
        nameArea.setDisabledTextColor(getForeground());
        coordArea = new JTextArea(roundDouble(entry.getLatitude(), 4) +
                                  ", " + roundDouble(entry.getLongitude(), 4));
        coordArea.setEnabled(false);
        coordArea.setDisabledTextColor(getForeground());
        setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, getForeground()));
        setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(1, 1, 1, 1);
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.LINE_START;
        gbc.gridwidth = 1;
        gbc.gridheight = 1;
        gbc.gridx = 0;
        gbc.gridy = 0;
        add(nameArea, gbc);
        gbc.gridy = 1;
        add(coordArea, gbc);
        addMouseListener(actionMouseAdapter);
        nameArea.addMouseListener(actionMouseAdapter);
        coordArea.addMouseListener(actionMouseAdapter);
    }

    @Override
    public void setBackground(Color bg) {
        super.setBackground(bg);
        if (nameArea != null) {
            nameArea.setBackground(bg);
            coordArea.setBackground(bg);
        }
    }

    @Override
    public void setForeground(Color fg) {
        super.setForeground(fg);
        if (nameArea != null) {
            nameArea.setDisabledTextColor(getForeground());
            coordArea.setDisabledTextColor(getForeground());
            nameArea.setForeground(fg);
            coordArea.setForeground(fg);
        }
    }

    public LocationEntry getEntry() {
        return entry;
    }

    private static String roundDouble(double d, int places) {
        return String.format("%." + places + "f", d);
    }

    public void addActionListener(ActionListener l) {
        actionListeners.add(l);
    }

    public void removeActionListener(ActionListener l) {
        actionListeners.remove(l);
    }

    private void fireActionEvent() {
        ActionEvent event = new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "selected");
        for (ActionListener l : actionListeners) {
            l.actionPerformed(event);
        }
    }
}
