package zander.ui.media.map;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.UIManager;

public class LocationEntryList extends JPanel {
    private static final long serialVersionUID = -5819587120501L;

    private final ArrayList<LocationSelectListener> locationSelectListeners = new ArrayList<LocationSelectListener>();
    private final ActionListener childListener = new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            if (e.getSource() instanceof LocationEntryComponent) {
                selected = (LocationEntryComponent) e.getSource();
                updateColors();
                fireLocationSelectEvent(selected.getEntry());
            }
        };
    };

    private Color selectionForeground;
    private Color selectionBackground;
    private LocationEntryComponent selected;

    public LocationEntryList() {
        setLayout(new ListLayout());
        selectionForeground = UIManager.getColor("List.selectionForeground");
        selectionBackground = UIManager.getColor("List.selectionBackground");
    }

    public void setSelectionForeground(Color c) {
        selectionForeground = c;
        updateColors();
    }

    public Color getSelectionForeground() {
        return selectionForeground;
    }

    public void setSelectionBackground(Color c) {
        selectionBackground = c;
        updateColors();
    }

    public Color getSelectionBackground() {
        return selectionBackground;
    }

    @Override
    public void setForeground(Color fg) {
        super.setForeground(fg);
        updateColors();
    }

    @Override
    public void setBackground(Color bg) {
        super.setBackground(bg);
        updateColors();
    }

    public void add(LocationEntry e) {
        LocationEntryComponent c = new LocationEntryComponent(e);
        c.setForeground(getForeground());
        c.setBackground(getBackground());
        c.addActionListener(childListener);
        add(c);
    }

    public void addAll(Iterable<LocationEntry> entries) {
        for (LocationEntry e : entries) {
            add(e);
        }
    }

    private void updateColors() {
        Color fg = getForeground();
        Color bg = getBackground();
        for (Component c : getComponents()) {
            if (c == selected) {
                c.setForeground(selectionForeground);
                c.setBackground(selectionBackground);
            } else{
                c.setForeground(fg);
                c.setBackground(bg);
            }
        }
        repaint();
    }

    private void fireLocationSelectEvent(LocationEntry entry) {
        LocationSelectEvent e = new LocationSelectEvent(this, entry);
        for (LocationSelectListener l : locationSelectListeners) {
            l.locationSelected(e);
        }
    }

    public void addLocationSelectListener(LocationSelectListener l) {
        locationSelectListeners.add(l);
    }

    public void removeLocationSelectListener(LocationSelectListener l) {
        locationSelectListeners.remove(l);
    }

    public void clearSelection() {
        if (selected != null) {
            selected.setForeground(getForeground());
            selected.setBackground(getBackground());
            selected = null;
            repaint();
        }
    }
}
