package zander.ui.media.map;

import java.awt.BorderLayout;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

import zander.ui.ErrorDialog;
import zander.ui.UIUtils;

public class LocationSelectPanel extends JPanel {
    private static final long serialVersionUID = -1824218931983L;

    private final ArrayList<LocationSelectListener> locationSelectListeners = new ArrayList<LocationSelectListener>();

    private final LocationEncoder encoder;
    private final LocationEntryList list;
    private final JScrollPane scroll;
    private final JLabel licenseLabel;

    public LocationSelectPanel(LocationEncoder encoder) {
        setLayout(new BorderLayout());
        this.encoder = encoder;
        list = new LocationEntryList();
        scroll = new JScrollPane(list);
        licenseLabel = new JLabel(encoder.getLicenseString(), SwingConstants.CENTER);
        licenseLabel.setFont(UIUtils.scaleFont(UIManager.getFont("Label.font"), 0.4f));
        add(scroll, BorderLayout.CENTER);
        add(licenseLabel, BorderLayout.PAGE_END);
    }

    public LocationEncoder getEncoder() {
        return encoder;
    }

    public void addActionListener(LocationSelectListener l) {
        locationSelectListeners.add(l);
    }

    public void removeActionListener(LocationSelectListener l) {
        locationSelectListeners.remove(l);
    }

    public boolean search(String query) {
        try {
            final List<LocationEntry> entries = encoder.encode(query);
            if (entries != null) {
                list.removeAll();
                list.addAll(entries);
                revalidate();
                repaint();
                return true;
            }
        } catch (IOException e) {
            ErrorDialog ed = new ErrorDialog("Search Error", "Search did not complete successfully.", e);
            ed.setVisible(true);
        }
        return false;
    }

    public void addLocationSelectListener(LocationSelectListener l) {
        list.addLocationSelectListener(l);
    }

    public void removeLocationSelectListener(LocationSelectListener l) {
        list.removeLocationSelectListener(l);
    }

    public void clearSelection() {
        list.clearSelection();
    }
}
